﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2019 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Scheduling.MRCPSP {

  [Item(nameof(Activity), "")]
  [StorableClass]
  public sealed class Activity : ParameterizedNamedItem {
    //public Activity()
    //    : this() {
    //}

    public override string ToString() => $"{Name} (Activity {Number})";

    [Storable]
    private readonly IFixedValueParameter<IntValue> numberParam;

    [Storable]
    private readonly IValueParameter<ItemList<Mode>> modesParam;

    [Storable]
    private readonly IValueParameter<ItemList<IntValue>> successorsParam;

    public Activity() {
      Parameters.Add(numberParam = new FixedValueParameter<IntValue>(nameof(Number), new IntValue()));
      Parameters.Add(modesParam = new ValueParameter<ItemList<Mode>>(nameof(Modes), new ItemList<Mode>()));
      Parameters.Add(successorsParam = new ValueParameter<ItemList<IntValue>>(nameof(Successors), new ItemList<IntValue>()));
    }

    [StorableConstructor]
    private Activity(bool deserializing)
        : base(deserializing) { }

    private Activity(Activity original, Cloner cloner)
        : base(original, cloner) {
      numberParam = cloner.Clone(original.numberParam);
      modesParam = cloner.Clone(original.modesParam);
      successorsParam = cloner.Clone(original.successorsParam);
    }

    public int Number {
      get => numberParam.Value.Value;
      set => numberParam.Value.Value = value;
    }

    public ItemList<Mode> Modes {
      get => modesParam.Value;
      set => modesParam.Value = value;
    }

    public ItemList<IntValue> Successors {
      get => successorsParam.Value;
      set => successorsParam.Value = value;
    }

    public override IDeepCloneable Clone(Cloner cloner) => new Activity(this, cloner);
  }
}
